#!/usr/bin/env ruby -Ku

require 'net/http'; Net::HTTP.version_1_2
require 'rexml/document'

module Atok_plugin
  AWS_ACCESS_KEY_ID = '1234567890ABCDEFGHIJ'      # あなたの「AWS Access Key ID」に置き換えてください。

  def run_process(request)
    doc = REXML::Document.new(fetch_amazon(request['composition_string']))

    candidates = doc.elements.to_a('/ItemSearchResponse/Items/Item').map do |item|
      candidate = {'hyoki' => '',
                   'comment_xhtml' => '',
                   'alternative' => '',
                   'alternative_type' => 'url_jump_string',
                   'alternative_alias' => ''}

      candidate['alternative'] << item.elements['DetailPageURL'].text

      title = ''
      price = ''
      publisher = ''
      creators = []
      date = ''

      item.elements.each('ItemAttributes/*') do |item_attribute|
        case item_attribute.name
        when 'Title'; title << item_attribute.text
        when 'ListPrice'; price << item_attribute.elements['FormattedPrice'].text
        when 'Publisher'; publisher << item_attribute.text
        when 'Creator'; creators << "(#{item_attribute.elements['@Role'].value})#{item_attribute.text}"
        when 'PublicationDate'; date << item_attribute.text
        when 'ReleaseDate'; date << item_attribute.text
        end
      end

      unless creators.empty?
        display_creators = %Q!<ul style="margin: .5em 0 0 1em;"><li>#{creators.join('</li><li>')}</li></ul>!
      else
        display_creators = ''
      end
      date = "(#{date})" unless date == ''

      image_element ||= item.elements['MediumImage']
      image_element ||= item.elements['SmallImage']
      image_element ||= item.elements['LargeImage']
      if image_element
        attr_values = [image_element.elements['URL'].text,
                       image_element.elements['Height'].text,
                       image_element.elements['Width'].text]
        image = ('<img src="%s" alt="image" height="%s" width="%s" />' % attr_values)
      else
        image = '<div>画像なし</div>'
      end

      candidate['hyoki'] << title
      candidate['alternative_alias'] << "#{title}の詳細"

      candidate['comment_xhtml'] = <<-EOT
        <?xml version="1.0" encoding="UTF-8" ?>
        <html xmlns="http://www.w3.org/1999/xhtml" xml:lang="ja" lang="ja">
          <head><title>AtokDirect</title></head>
          <body>
            <div style="font-size: 1.1em; font-weight: bold;">#{title}</div>
            <div>#{image}<br />#{publisher}#{date}<br />#{price}</div>
            #{display_creators}
          </body>
        </html>
      EOT

      candidate
    end

    {'candidate' => candidates}
  rescue TimeoutError => evar
    nil
  rescue => evar
    nil
  end

  private
  def fetch_amazon(keywords)
    url = 'http://ecs.amazonaws.jp/onca/xml?Service=AWSECommerceService&Version=2008-10-07' <<
          "&AWSAccessKeyId=#{AWS_ACCESS_KEY_ID}" <<
          "&Operation=ItemSearch" <<
          "&SearchIndex=Blended" <<
          "&ResponseGroup=ItemAttributes,Images" <<
          "&Keywords=#{URI.encode(keywords)}"
    http_response = Net::HTTP.get_response(URI.parse(url))
    unless http_response.is_a?(Net::HTTPOK)
      raise 'Invalid http response.'
    else
      http_response.body
    end
  end
end
